from langchain_community.utilities.redis import (
    TokenEscaper,
    check_redis_module_exist,
    get_client,
)

__all__ = [
    "TokenEscaper",
    "check_redis_module_exist",
    "get_client",
]



"""**Document Loaders**  are classes to load Documents.

**Document Loaders** are usually used to load a lot of Documents in a single run.

**Class hierarchy:**

.. code-block::

    BaseLoader --> <name>Loader  # Examples: TextLoader, UnstructuredFileLoader

**Main helpers:**

.. code-block::

    Document, <name>TextSplitter
"""
import warnings
from typing import Any

from langchain_core._api import LangChainDeprecationWarning

from langchain.utils.interactive_env import is_interactive_env

# For backwards compatibility
_old_to_new_name = {
    "PagedPDFSplitter": "PyPDFLoader",
    "TelegramChatLoader": "TelegramChatFileLoader",
}


def __getattr__(name: str) -> Any:
    from langchain_community import document_loaders

    # If not in interactive env, raise warning.
    if not is_interactive_env():
        warnings.warn(
            "Importing document loaders from langchain is deprecated. Importing from "
            "langchain will no longer be supported as of langchain==0.2.0. "
            "Please import from langchain-community instead:\n\n"
            f"`from langchain_community.document_loaders import {name}`.\n\n"
            "To install gigachain-community run `pip install -U gigachain-community`.",
            category=LangChainDeprecationWarning,
        )

    if name in _old_to_new_name:
        warnings.warn(
            f"Using legacy class name {name}, use {_old_to_new_name[name]} instead."
        )
        name = _old_to_new_name[name]

    return getattr(document_loaders, name)


__all__ = [
    "AcreomLoader",
    "AsyncHtmlLoader",
    "AsyncChromiumLoader",
    "AZLyricsLoader",
    "AcreomLoader",
    "AirbyteCDKLoader",
    "AirbyteGongLoader",
    "AirbyteJSONLoader",
    "AirbyteHubspotLoader",
    "AirbyteSalesforceLoader",
    "AirbyteShopifyLoader",
    "AirbyteStripeLoader",
    "AirbyteTypeformLoader",
    "AirbyteZendeskSupportLoader",
    "AirtableLoader",
    "AmazonTextractPDFLoader",
    "ApifyDatasetLoader",
    "ArcGISLoader",
    "ArxivLoader",
    "AssemblyAIAudioTranscriptLoader",
    "AsyncHtmlLoader",
    "AzureAIDataLoader",
    "AzureBlobStorageContainerLoader",
    "AzureBlobStorageFileLoader",
    "BSHTMLLoader",
    "BibtexLoader",
    "BigQueryLoader",
    "BiliBiliLoader",
    "BlackboardLoader",
    "Blob",
    "BlobLoader",
    "BlockchainDocumentLoader",
    "BraveSearchLoader",
    "BrowserlessLoader",
    "CSVLoader",
    "ChatGPTLoader",
    "CoNLLULoader",
    "CollegeConfidentialLoader",
    "ConcurrentLoader",
    "ConfluenceLoader",
    "CouchbaseLoader",
    "CubeSemanticLoader",
    "DataFrameLoader",
    "DatadogLogsLoader",
    "DiffbotLoader",
    "DirectoryLoader",
    "DiscordChatLoader",
    "DocugamiLoader",
    "DocusaurusLoader",
    "Docx2txtLoader",
    "DropboxLoader",
    "DuckDBLoader",
    "EtherscanLoader",
    "EverNoteLoader",
    "FacebookChatLoader",
    "FaunaLoader",
    "FigmaFileLoader",
    "FileSystemBlobLoader",
    "GCSDirectoryLoader",
    "GCSFileLoader",
    "GeoDataFrameLoader",
    "GitHubIssuesLoader",
    "GitLoader",
    "GitbookLoader",
    "GoogleApiClient",
    "GoogleApiYoutubeLoader",
    "GoogleSpeechToTextLoader",
    "GoogleDriveLoader",
    "GutenbergLoader",
    "HNLoader",
    "HuggingFaceDatasetLoader",
    "IFixitLoader",
    "IMSDbLoader",
    "ImageCaptionLoader",
    "IuguLoader",
    "JSONLoader",
    "JoplinLoader",
    "LarkSuiteDocLoader",
    "LakeFSLoader",
    "MHTMLLoader",
    "MWDumpLoader",
    "MastodonTootsLoader",
    "MathpixPDFLoader",
    "MaxComputeLoader",
    "MergedDataLoader",
    "ModernTreasuryLoader",
    "MongodbLoader",
    "NewsURLLoader",
    "NotebookLoader",
    "NotionDBLoader",
    "NotionDirectoryLoader",
    "OBSDirectoryLoader",
    "OBSFileLoader",
    "ObsidianLoader",
    "OneDriveFileLoader",
    "OneDriveLoader",
    "OnlinePDFLoader",
    "OpenCityDataLoader",
    "OutlookMessageLoader",
    "PDFMinerLoader",
    "PDFMinerPDFasHTMLLoader",
    "PDFPlumberLoader",
    "PagedPDFSplitter",
    "PlaywrightURLLoader",
    "PolarsDataFrameLoader",
    "PsychicLoader",
    "PubMedLoader",
    "PyMuPDFLoader",
    "PyPDFDirectoryLoader",
    "PyPDFLoader",
    "PyPDFium2Loader",
    "PySparkDataFrameLoader",
    "PythonLoader",
    "RSSFeedLoader",
    "ReadTheDocsLoader",
    "RecursiveUrlLoader",
    "RedditPostsLoader",
    "RoamLoader",
    "RocksetLoader",
    "S3DirectoryLoader",
    "S3FileLoader",
    "SRTLoader",
    "SeleniumURLLoader",
    "SharePointLoader",
    "SitemapLoader",
    "SlackDirectoryLoader",
    "SnowflakeLoader",
    "SpreedlyLoader",
    "StripeLoader",
    "TelegramChatApiLoader",
    "TelegramChatFileLoader",
    "TelegramChatLoader",
    "TensorflowDatasetLoader",
    "TencentCOSDirectoryLoader",
    "TencentCOSFileLoader",
    "TextLoader",
    "ToMarkdownLoader",
    "TomlLoader",
    "TrelloLoader",
    "TwitterTweetLoader",
    "UnstructuredAPIFileIOLoader",
    "UnstructuredAPIFileLoader",
    "UnstructuredCSVLoader",
    "UnstructuredEPubLoader",
    "UnstructuredEmailLoader",
    "UnstructuredExcelLoader",
    "UnstructuredFileIOLoader",
    "UnstructuredFileLoader",
    "UnstructuredHTMLLoader",
    "UnstructuredImageLoader",
    "UnstructuredMarkdownLoader",
    "UnstructuredODTLoader",
    "UnstructuredOrgModeLoader",
    "UnstructuredPDFLoader",
    "UnstructuredPowerPointLoader",
    "UnstructuredRSTLoader",
    "UnstructuredRTFLoader",
    "UnstructuredTSVLoader",
    "UnstructuredURLLoader",
    "UnstructuredWordDocumentLoader",
    "UnstructuredXMLLoader",
    "WeatherDataLoader",
    "WebBaseLoader",
    "WhatsAppChatLoader",
    "WikipediaLoader",
    "XorbitsLoader",
    "YoutubeAudioLoader",
    "YoutubeLoader",
]



"""**Utilities** are the integrations with third-part systems and packages.

Other LangChain classes use **Utilities** to interact with third-part systems
and packages.
"""
import warnings
from typing import Any

from langchain_community.utilities.requests import (
    Requests,
    RequestsWrapper,
    TextRequestsWrapper,
)
from langchain_core._api import LangChainDeprecationWarning

from langchain.utils.interactive_env import is_interactive_env


def __getattr__(name: str) -> Any:
    from langchain_community import utilities

    # If not in interactive env, raise warning.
    if not is_interactive_env():
        warnings.warn(
            "Importing this utility from langchain is deprecated. Importing it from "
            "langchain will no longer be supported as of langchain==0.2.0. "
            "Please import from langchain-community instead:\n\n"
            f"`from langchain_community.utilities import {name}`.\n\n"
            "To install gigachain-community run `pip install -U gigachain-community`.",
            category=LangChainDeprecationWarning,
        )

    return getattr(utilities, name)


__all__ = [
    "AlphaVantageAPIWrapper",
    "ApifyWrapper",
    "ArceeWrapper",
    "ArxivAPIWrapper",
    "BibtexparserWrapper",
    "BingSearchAPIWrapper",
    "BraveSearchWrapper",
    "DuckDuckGoSearchAPIWrapper",
    "GoldenQueryAPIWrapper",
    "GoogleFinanceAPIWrapper",
    "GoogleLensAPIWrapper",
    "GoogleJobsAPIWrapper",
    "GooglePlacesAPIWrapper",
    "GoogleScholarAPIWrapper",
    "GoogleTrendsAPIWrapper",
    "GoogleSearchAPIWrapper",
    "GoogleSerperAPIWrapper",
    "GraphQLAPIWrapper",
    "JiraAPIWrapper",
    "LambdaWrapper",
    "MaxComputeAPIWrapper",
    "MerriamWebsterAPIWrapper",
    "MetaphorSearchAPIWrapper",
    "NasaAPIWrapper",
    "OpenWeatherMapAPIWrapper",
    "OutlineAPIWrapper",
    "Portkey",
    "PowerBIDataset",
    "PubMedAPIWrapper",
    "PythonREPL",
    "Requests",
    "RequestsWrapper",
    "SteamWebAPIWrapper",
    "SQLDatabase",
    "SceneXplainAPIWrapper",
    "SearchApiAPIWrapper",
    "SearxSearchWrapper",
    "SerpAPIWrapper",
    "SparkSQL",
    "StackExchangeAPIWrapper",
    "TensorflowDatasets",
    "TextRequestsWrapper",
    "TwilioAPIWrapper",
    "WikipediaAPIWrapper",
    "WolframAlphaAPIWrapper",
    "ZapierNLAWrapper",
]



from langchain_community.vectorstores.elasticsearch import (
    ApproxRetrievalStrategy,
    BaseRetrievalStrategy,
    ElasticsearchStore,
    ExactRetrievalStrategy,
    SparseRetrievalStrategy,
)

__all__ = [
    "BaseRetrievalStrategy",
    "ApproxRetrievalStrategy",
    "ExactRetrievalStrategy",
    "SparseRetrievalStrategy",
    "ElasticsearchStore",
]



from langchain_community.tools.sql_database.tool import (
    BaseSQLDatabaseTool,
    InfoSQLDatabaseTool,
    ListSQLDatabaseTool,
    QuerySQLCheckerTool,
    QuerySQLDataBaseTool,
)

__all__ = [
    "BaseSQLDatabaseTool",
    "QuerySQLDataBaseTool",
    "InfoSQLDatabaseTool",
    "ListSQLDatabaseTool",
    "QuerySQLCheckerTool",
]



# flake8: noqa
from langchain_core.prompts.prompt import PromptTemplate

_DEFAULT_ENTITY_EXTRACTION_TEMPLATE = """Извлеките все сущности из следующего текста. В качестве руководства, собственное имя обычно пишется с заглавной буквы. Вы должны обязательно извлечь все имена и места.

Верните результат в виде одного списка, разделенного запятыми, или NONE, если нет ничего интересного для возврата.

ПРИМЕР
Я пытаюсь улучшить интерфейсы Langchain, UX, его интеграции с различными продуктами, которые может захотеть пользователь ... много всего.
Вывод: Langchain
КОНЕЦ ПРИМЕРА

ПРИМЕР
Я пытаюсь улучшить интерфейсы Langchain, UX, его интеграции с различными продуктами, которые может захотеть пользователь ... много всего. Я работаю с Сэмом.
Вывод: Langchain, Сэм
КОНЕЦ ПРИМЕРА

Начните!

{input}
Вывод:"""
ENTITY_EXTRACTION_PROMPT = PromptTemplate(
    input_variables=["input"], template=_DEFAULT_ENTITY_EXTRACTION_TEMPLATE
)

_DEFAULT_GRAPH_QA_TEMPLATE = """Используйте следующие тройки знаний, чтобы ответить на вопрос в конце. Если вы не знаете ответа, просто скажите, что не знаете, не пытайтесь придумать ответ.

{context}

Question: {question}
Helpful Answer:"""
GRAPH_QA_PROMPT = PromptTemplate(
    template=_DEFAULT_GRAPH_QA_TEMPLATE, input_variables=["context", "question"]
)

CYPHER_GENERATION_TEMPLATE = """Задача: Сгенерировать выражение Cypher для запроса к графовой базе данных.
Инструкции:
Используйте только предоставленные типы отношений и свойства в схеме.
Не используйте другие типы отношений или свойства, которые не предоставлены.
Схема:
{schema}
Примечание: Не включайте в ответ никаких пояснений или извинений.
Не отвечайте на вопросы, которые могут просить о чем-то, кроме создания выражения Cypher.
Не включайте в ответ никакой текст, кроме сгенерированного выражения Cypher.

Question:
{question}"""
CYPHER_GENERATION_PROMPT = PromptTemplate(
    input_variables=["schema", "question"], template=CYPHER_GENERATION_TEMPLATE
)

NEBULAGRAPH_EXTRA_INSTRUCTIONS = """
Инструкции:

Сначала сгенерируйте выражение Cypher, а затем преобразуйте его в диалект NebulaGraph Cypher (а не стандартный):
1. требуется явное указание метки только при ссылке на свойства узла: v.`Foo`.name
2. обратите внимание, что явное указание метки не требуется для свойств ребра, поэтому это e.name вместо e.`Bar`.name
3. для сравнения используется двойной знак равенства: `==` вместо `=`
Например:
```diff
< MATCH (p:person)-[e:directed]->(m:movie) WHERE m.name = 'The Godfather II'
< RETURN p.name, e.year, m.name;
---
> MATCH (p:`person`)-[e:directed]->(m:`movie`) WHERE m.`movie`.`name` == 'The Godfather II'
> RETURN p.`person`.`name`, e.year, m.`movie`.`name`;
```\n"""

NGQL_GENERATION_TEMPLATE = CYPHER_GENERATION_TEMPLATE.replace(
    "Generate Cypher", "Generate NebulaGraph Cypher"
).replace("Instructions:", NEBULAGRAPH_EXTRA_INSTRUCTIONS)

NGQL_GENERATION_PROMPT = PromptTemplate(
    input_variables=["schema", "question"], template=NGQL_GENERATION_TEMPLATE
)

KUZU_EXTRA_INSTRUCTIONS = """
Инструкции:

Сгенерируйте выражение с диалектом Kùzu Cypher (а не стандартным):
1. не используйте оператор `WHERE EXISTS` для проверки наличия свойства, потому что у базы данных Kùzu есть фиксированная схема.
2. не опускайте шаблон отношения. Всегда используйте `()-[]->()` вместо `()->()`.
3. не включайте никаких замечаний или комментариев, даже если выражение не дает ожидаемого результата.
```\n"""

KUZU_GENERATION_TEMPLATE = CYPHER_GENERATION_TEMPLATE.replace(
    "Generate Cypher", "Generate Kùzu Cypher"
).replace("Instructions:", KUZU_EXTRA_INSTRUCTIONS)

KUZU_GENERATION_PROMPT = PromptTemplate(
    input_variables=["schema", "question"], template=KUZU_GENERATION_TEMPLATE
)

GREMLIN_GENERATION_TEMPLATE = CYPHER_GENERATION_TEMPLATE.replace("Cypher", "Gremlin")

GREMLIN_GENERATION_PROMPT = PromptTemplate(
    input_variables=["schema", "question"], template=GREMLIN_GENERATION_TEMPLATE
)

CYPHER_QA_TEMPLATE = """Вы - помощник, который помогает формировать понятные и человекочитаемые ответы.
Часть с информацией содержит предоставленную информацию, которую вы должны использовать для составления ответа.
Предоставленная информация является авторитетной, вы никогда не должны сомневаться в ней или пытаться использовать свои внутренние знания для ее корректировки.
Сделайте ответ звучать как ответ на вопрос. Не упоминайте, что вы основываетесь на предоставленной информации.
Если предоставленная информация пуста, скажите, что не знаете ответа.
Информация:
"""

CYPHER_QA_TEMPLATE = """ы - помощник, который помогает формировать понятные и человекочитаемые ответы.
Часть с информацией содержит предоставленную информацию, которую вы должны использовать для составления ответа.
Предоставленная информация является авторитетной, вы никогда не должны сомневаться в ней или пытаться использовать свои внутренние знания для ее корректировки.
Сделайте ответ звучать как ответ на вопрос. Не упоминайте, что вы основываетесь на предоставленной информации.
Если предоставленная информация пуста, скажите, что не знаете ответа.
Пример:

Question: Какие менеджеры владеют акциями Neo4j?
Context:[manager:CTL LLC, manager:JANE STREET GROUP LLC]
Helpful Answer: CTL LLC, JANE STREET GROUP LLC владеют акциями Neo4j.

Следуйте этому примеру при генерации ответов.
Если предоставленная информация пуста, скажите, что не знаете ответа.
Context:{context}
Question: {question}
Helpful Answer:"""
CYPHER_QA_PROMPT = PromptTemplate(
    input_variables=["context", "question"], template=CYPHER_QA_TEMPLATE
)

SPARQL_INTENT_TEMPLATE = """Задача: Определить намерение запроса SPARQL по промпту и вернуть соответствующий тип запроса SPARQL.
Вы - помощник, который различает разные типы запросов и возвращает соответствующие типы запросов SPARQL.
Учитывайте только следующие типы запросов:
* SELECT: этот тип запроса соответствует вопросам
* UPDATE: этот тип запроса соответствует всем запросам на удаление, вставку или изменение троек
Примечание: Будьте максимально краткими.
Не включайте в ответ никаких пояснений или извинений.
Не отвечайте на вопросы, которые просят о чем-то, кроме определения типа запроса SPARQL.
Не включайте никаких ненужных пробелов или текста, кроме типа запроса, то есть либо верните 'SELECT', либо 'UPDATE'.

Промпт:
{prompt}
Helpful Answer:"""
SPARQL_INTENT_PROMPT = PromptTemplate(
    input_variables=["prompt"], template=SPARQL_INTENT_TEMPLATE
)

SPARQL_GENERATION_SELECT_TEMPLATE = """Задача: Сгенерировать выражение SPARQL SELECT для запроса к графовой базе данных.
Например, чтобы найти все адреса электронной почты Джона Доу, следующий запрос в обратных кавычках будет подходящим:
```
PREFIX foaf: <http://xmlns.com/foaf/0.1/>
SELECT ?email
WHERE {{
    ?person foaf:name "John Doe" .
    ?person foaf:mbox ?email .
}}
```
Инструкции:
Используйте только типы узлов и свойства, предоставленные в схеме.
Не используйте никакие типы узлов и свойства, которые не являются явно предоставленными.
Включите все необходимые префиксы.
Схема:
{schema}
Примечание: Будьте максимально краткими.
Не включайте в ответ никаких пояснений или извинений.
Не отвечайте на вопросы, которые просят о чем-то, кроме создания запроса SPARQL.
Не включайте никакой текст, кроме сгенерированного запроса SPARQL.

Question:
{prompt}"""
SPARQL_GENERATION_SELECT_PROMPT = PromptTemplate(
    input_variables=["schema", "prompt"], template=SPARQL_GENERATION_SELECT_TEMPLATE
)

SPARQL_GENERATION_UPDATE_TEMPLATE = """Задача: Сгенерировать выражение SPARQL UPDATE для обновления графовой базы данных.
Например, чтобы добавить 'jane.doe@foo.bar' в качестве нового адреса электронной почты для Джейн Доу, следующий запрос в обратных кавычках будет подходящим:
```
PREFIX foaf: <http://xmlns.com/foaf/0.1/>
INSERT {{
    ?person foaf:mbox <mailto:jane.doe@foo.bar> .
}}
WHERE {{
    ?person foaf:name "Jane Doe" .
}}
```
Инструкции:
Сделайте запрос как можно короче и избегайте добавления ненужных троек.
Используйте только типы узлов и свойства, предоставленные в схеме.
Не используйте никакие типы узлов и свойства, которые не являются явно предоставленными.
Включите все необходимые префиксы.
Схема:
{schema}
Примечание: Будьте максимально краткими.
Не включайте в ответ никаких пояснений или извинений.
Не отвечайте на вопросы, которые просят о чем-то, кроме создания запроса SPARQL.
Верните только сгенерированный запрос SPARQL, ничего больше.

Информация для вставки:
{prompt}"""
SPARQL_GENERATION_UPDATE_PROMPT = PromptTemplate(
    input_variables=["schema", "prompt"], template=SPARQL_GENERATION_UPDATE_TEMPLATE
)

SPARQL_QA_TEMPLATE = """Задача: Сгенерировать естественноязыковой ответ на основе результатов запроса SPARQL.
Вы - помощник, который создает хорошо написанные и понятные человеку ответы.
Часть с информацией содержит предоставленную информацию, которую вы можете использовать для составления ответа.
Предоставленная информация является авторитетной, вы никогда не должны сомневаться в ней или пытаться использовать свои внутренние знания для ее корректировки.
Сделайте ответ звучать так, как будто информация поступает от помощника по искусственному интеллекту, но не добавляйте никакую информацию.
Информация:
{context}

Question: {prompt}
Helpful Answer:"""
SPARQL_QA_PROMPT = PromptTemplate(
    input_variables=["context", "prompt"], template=SPARQL_QA_TEMPLATE
)

GRAPHDB_SPARQL_GENERATION_TEMPLATE = """
Write a SPARQL SELECT query for querying a graph database.
The ontology schema delimited by triple backticks in Turtle format is:
```
{schema}
```
Use only the classes and properties provided in the schema to construct the SPARQL query.
Do not use any classes or properties that are not explicitly provided in the SPARQL query.
Include all necessary prefixes.
Do not include any explanations or apologies in your responses.
Do not wrap the query in backticks.
Do not include any text except the SPARQL query generated.
The question delimited by triple backticks is:
```
{prompt}
```
"""
GRAPHDB_SPARQL_GENERATION_PROMPT = PromptTemplate(
    input_variables=["schema", "prompt"],
    template=GRAPHDB_SPARQL_GENERATION_TEMPLATE,
)

GRAPHDB_SPARQL_FIX_TEMPLATE = """
This following SPARQL query delimited by triple backticks
```
{generated_sparql}
```
is not valid.
The error delimited by triple backticks is
```
{error_message}
```
Give me a correct version of the SPARQL query.
Do not change the logic of the query.
Do not include any explanations or apologies in your responses.
Do not wrap the query in backticks.
Do not include any text except the SPARQL query generated.
The ontology schema delimited by triple backticks in Turtle format is:
```
{schema}
```
"""

GRAPHDB_SPARQL_FIX_PROMPT = PromptTemplate(
    input_variables=["error_message", "generated_sparql", "schema"],
    template=GRAPHDB_SPARQL_FIX_TEMPLATE,
)

GRAPHDB_QA_TEMPLATE = """Task: Generate a natural language response from the results of a SPARQL query.
You are an assistant that creates well-written and human understandable answers.
The information part contains the information provided, which you can use to construct an answer.
The information provided is authoritative, you must never doubt it or try to use your internal knowledge to correct it.
Make your response sound like the information is coming from an AI assistant, but don't add any information.
Don't use internal knowledge to answer the question, just say you don't know if no information is available.
Information:
{context}

Question: {prompt}
Helpful Answer:"""
GRAPHDB_QA_PROMPT = PromptTemplate(
    input_variables=["context", "prompt"], template=GRAPHDB_QA_TEMPLATE
)

AQL_GENERATION_TEMPLATE = """Задача: Сгенерировать запрос на языке ArangoDB Query Language (AQL) из пользовательского ввода.

Вы - эксперт по языку запросов ArangoDB Query Language (AQL), ответственный за перевод `Пользовательского ввода` в запрос на языке ArangoDB Query Language (AQL).

Вам предоставлена `Схема ArangoDB`. Это JSON-объект, содержащий:
1. `Схему графа`: Список всех графов в базе данных ArangoDB вместе с их отношениями ребер.
2. `Схему коллекции`: Список всех коллекций в базе данных ArangoDB вместе с их свойствами документов/ребер и примером документа/ребра.

Вам также могут быть предоставлены наборы `Примеров запросов AQL` для помощи в создании `Запроса AQL`. Если они предоставлены, `Примеры запросов AQL` должны использоваться в качестве справочного материала, аналогично тому, как должна использоваться `Схема ArangoDB`.

Что вы должны делать:
- Думайте поэтапно.
- Ориентируйтесь на `Схему ArangoDB` и `Примеры запросов AQL` (если они предоставлены), чтобы сгенерировать запрос.
- Начните `Запрос AQL` с ключевого слова `WITH`, чтобы указать все необходимые коллекции ArangoDB.
- Верните `Запрос AQL`, заключенный в 3 обратные кавычки (```).
- Используйте только предоставленные типы отношений и свойства в `Схеме ArangoDB` и любых запросах `Примеров запросов AQL`.
- Отвечайте только на запросы, связанные с генерацией запроса AQL.
- Если запрос не связан с генерацией запроса AQL, скажите, что вы не можете помочь пользователю.

Что вы не должны делать:
- Не используйте свойства/отношения, которые нельзя вывести из `Схемы ArangoDB` или `Примеров запросов AQL`. 
- Не включайте никакой текст, кроме сгенерированного запроса AQL.
- Не предоставляйте пояснений или извинений в своих ответах.
- Не генерируйте запрос AQL, который удаляет или изменяет какие-либо данные.

Под никаким предлогом не генерируйте запрос AQL, который удаляет какие-либо данные.

Схема ArangoDB:
{adb_schema}

Примеры запросов AQL (Необязательно):
{aql_examples}

Пользовательский ввод:
{user_input}

Запрос AQL: 
"""

AQL_GENERATION_PROMPT = PromptTemplate(
    input_variables=["adb_schema", "aql_examples", "user_input"],
    template=AQL_GENERATION_TEMPLATE,
)

AQL_FIX_TEMPLATE = """Задача: Исправить сообщение об ошибке языка запросов ArangoDB (AQL) запроса на языке ArangoDB Query Language.

Вы - эксперт по языку запросов ArangoDB Query Language (AQL), ответственный за исправление предоставленного `Запроса AQL` на основе предоставленной `Ошибки AQL`. 

`Ошибкой AQL` объясняется, почему `Запрос AQL` не может быть выполнен в базе данных.
`Ошибкой AQL` также может быть указано положение ошибки относительно общего количества строк `Запроса AQL`.
Например, 'ошибка X в позиции 2:5' означает, что ошибка X происходит в строке 2, столбце 5 `Запроса AQL`.  

Вам также предоставлена `Схема ArangoDB`. Это JSON-объект, содержащий:
1. `Схему графа`: Список всех графов в базе данных ArangoDB вместе с их отношениями ребер.
2. `Схему коллекции`: Список всех коллекций в базе данных ArangoDB вместе с их свойствами документов/ребер и примером документа/ребра.

Вы должны вывести `Исправленный запрос AQL`, заключенный в 3 обратные кавычки (```). Не включайте никакой текст, кроме Исправленного запроса AQL.

Не забывайте думать поэтапно.

Схема ArangoDB:
{adb_schema}

Запрос AQL:
{aql_query}

Ошибка AQL:
{aql_error}

Исправленный запрос AQL:
"""

AQL_FIX_PROMPT = PromptTemplate(
    input_variables=[
        "adb_schema",
        "aql_query",
        "aql_error",
    ],
    template=AQL_FIX_TEMPLATE,
)

AQL_QA_TEMPLATE = """Задача: Сгенерировать естественноязыковое `Summary` на основе результатов запроса на языке ArangoDB Query Language.

Вы - эксперт по языку запросов ArangoDB Query Language (AQL), ответственный за создание хорошо написанного `Summary` на основе `Пользовательского ввода` и связанного `Результата AQL`.

Пользователь выполнил запрос на языке ArangoDB Query Language, который вернул результат AQL в формате JSON.
Вам предстоит создать `Summary` на основе результата AQL.

Вам предоставлена следующая информация:
- `Схема ArangoDB`: содержит схему базы данных ArangoDB пользователя.
- `Пользовательский ввод`: исходный вопрос/запрос пользователя, который был преобразован в запрос на языке ArangoDB Query Language.
- `Запрос AQL`: эквивалент запроса на языке AQL для `Пользовательского ввода`, переведенный другой моделью искусственного интеллекта. Если вы считаете его некорректным, предложите другой запрос AQL.
- `Результат AQL`: JSON-вывод, возвращенный выполнением `Запроса AQL` в базе данных ArangoDB.

Не забывайте думать поэтапно.

Ваше `Summary` должно звучать так, как будто это ответ на `Пользовательский ввод`.
Ваше `Summary` не должно содержать никаких упоминаний о `Запросе AQL` или `Результате AQL`.

Схема ArangoDB:
{adb_schema}

Пользовательский ввод:
{user_input}

Запрос AQL:
{aql_query}

Результат AQL:
{aql_result}
"""
AQL_QA_PROMPT = PromptTemplate(
    input_variables=["adb_schema", "user_input", "aql_query", "aql_result"],
    template=AQL_QA_TEMPLATE,
)


NEPTUNE_OPENCYPHER_EXTRA_INSTRUCTIONS = """
Инструкции:
Сгенерируйте запрос в формате openCypher и следуйте этим правилам:
Не используйте предикатные функции `NONE`, `ALL` или `ANY`, вместо этого используйте генерацию списков.
Не используйте функцию `REDUCE`. Вместо этого используйте комбинацию генерации списков и оператора `UNWIND`, чтобы достичь аналогичных результатов.
Не используйте оператор `FOREACH`. Вместо этого используйте комбинацию операторов `WITH` и `UNWIND`, чтобы достичь аналогичных результатов.{extra_instructions}
\n"""

NEPTUNE_OPENCYPHER_GENERATION_TEMPLATE = CYPHER_GENERATION_TEMPLATE.replace(
    "Instructions:", NEPTUNE_OPENCYPHER_EXTRA_INSTRUCTIONS
)

NEPTUNE_OPENCYPHER_GENERATION_PROMPT = PromptTemplate(
    input_variables=["schema", "question", "extra_instructions"],
    template=NEPTUNE_OPENCYPHER_GENERATION_TEMPLATE,
)

NEPTUNE_OPENCYPHER_GENERATION_SIMPLE_TEMPLATE = """
Write an openCypher query to answer the following question. Do not explain the answer. Only return the query.{extra_instructions}
Question:  "{question}". 
Here is the property graph schema: 
{schema}
\n"""

NEPTUNE_OPENCYPHER_GENERATION_SIMPLE_PROMPT = PromptTemplate(
    input_variables=["schema", "question", "extra_instructions"],
    template=NEPTUNE_OPENCYPHER_GENERATION_SIMPLE_TEMPLATE,
)



"""**Embedding models**  are wrappers around embedding models
from different APIs and services.

**Embedding models** can be LLMs or not.

**Class hierarchy:**

.. code-block::

    Embeddings --> <name>Embeddings  # Examples: OpenAIEmbeddings, HuggingFaceEmbeddings
"""


import logging
import warnings
from typing import Any

from langchain_core._api import LangChainDeprecationWarning

from langchain.embeddings.cache import CacheBackedEmbeddings
from langchain.utils.interactive_env import is_interactive_env


def __getattr__(name: str) -> Any:
    from langchain_community import embeddings

    # If not in interactive env, raise warning.
    if not is_interactive_env():
        warnings.warn(
            "Importing embeddings from langchain is deprecated. Importing from "
            "langchain will no longer be supported as of langchain==0.2.0. "
            "Please import from langchain-community instead:\n\n"
            f"`from langchain_community.embeddings import {name}`.\n\n"
            "To install gigachain-community run `pip install -U gigachain-community`.",
            category=LangChainDeprecationWarning,
        )

    return getattr(embeddings, name)


logger = logging.getLogger(__name__)

__all__ = [
    "OpenAIEmbeddings",
    "AzureOpenAIEmbeddings",
    "CacheBackedEmbeddings",
    "ClarifaiEmbeddings",
    "CohereEmbeddings",
    "DatabricksEmbeddings",
    "ElasticsearchEmbeddings",
    "FastEmbedEmbeddings",
    "HuggingFaceEmbeddings",
    "HuggingFaceInferenceAPIEmbeddings",
    "InfinityEmbeddings",
    "GradientEmbeddings",
    "JinaEmbeddings",
    "LlamaCppEmbeddings",
    "HuggingFaceHubEmbeddings",
    "MlflowEmbeddings",
    "MlflowAIGatewayEmbeddings",
    "ModelScopeEmbeddings",
    "TensorflowHubEmbeddings",
    "SagemakerEndpointEmbeddings",
    "HuggingFaceInstructEmbeddings",
    "MosaicMLInstructorEmbeddings",
    "SelfHostedEmbeddings",
    "SelfHostedHuggingFaceEmbeddings",
    "SelfHostedHuggingFaceInstructEmbeddings",
    "FakeEmbeddings",
    "DeterministicFakeEmbedding",
    "AlephAlphaAsymmetricSemanticEmbedding",
    "AlephAlphaSymmetricSemanticEmbedding",
    "SentenceTransformerEmbeddings",
    "GooglePalmEmbeddings",
    "MiniMaxEmbeddings",
    "VertexAIEmbeddings",
    "BedrockEmbeddings",
    "DeepInfraEmbeddings",
    "EdenAiEmbeddings",
    "DashScopeEmbeddings",
    "EmbaasEmbeddings",
    "OctoAIEmbeddings",
    "SpacyEmbeddings",
    "NLPCloudEmbeddings",
    "GPT4AllEmbeddings",
    "XinferenceEmbeddings",
    "LocalAIEmbeddings",
    "AwaEmbeddings",
    "HuggingFaceBgeEmbeddings",
    "ErnieEmbeddings",
    "JavelinAIGatewayEmbeddings",
    "OllamaEmbeddings",
    "QianfanEmbeddingsEndpoint",
    "JohnSnowLabsEmbeddings",
    "VoyageEmbeddings",
    "BookendEmbeddings",
]


# TODO: this is in here to maintain backwards compatibility
class HypotheticalDocumentEmbedder:
    def __init__(self, *args: Any, **kwargs: Any):
        logger.warning(
            "Using a deprecated class. Please use "
            "`from langchain.chains import HypotheticalDocumentEmbedder` instead"
        )
        from langchain.chains.hyde.base import HypotheticalDocumentEmbedder as H

        return H(*args, **kwargs)  # type: ignore

    @classmethod
    def from_llm(cls, *args: Any, **kwargs: Any) -> Any:
        logger.warning(
            "Using a deprecated class. Please use "
            "`from langchain.chains import HypotheticalDocumentEmbedder` instead"
        )
        from langchain.chains.hyde.base import HypotheticalDocumentEmbedder as H

        return H.from_llm(*args, **kwargs)



from langchain_community.callbacks.human import (
    AsyncHumanApprovalCallbackHandler,
    HumanApprovalCallbackHandler,
    HumanRejectedException,
)

__all__ = [
    "HumanRejectedException",
    "HumanApprovalCallbackHandler",
    "AsyncHumanApprovalCallbackHandler",
]



from typing import Any, List, Sequence, Tuple, Union

from langchain_core._api import deprecated
from langchain_core.agents import AgentAction, AgentFinish
from langchain_core.callbacks import Callbacks
from langchain_core.language_models import BaseLanguageModel
from langchain_core.prompts.base import BasePromptTemplate
from langchain_core.prompts.chat import AIMessagePromptTemplate, ChatPromptTemplate
from langchain_core.runnables import Runnable, RunnablePassthrough
from langchain_core.tools import BaseTool

from langchain.agents.agent import BaseSingleActionAgent
from langchain.agents.format_scratchpad import format_xml
from langchain.agents.output_parsers import XMLAgentOutputParser
from langchain.agents.xml.prompt import agent_instructions
from langchain.chains.llm import LLMChain
from langchain.tools.render import ToolsRenderer, render_text_description


@deprecated("0.1.0", alternative="create_xml_agent", removal="0.2.0")
class XMLAgent(BaseSingleActionAgent):
    """Agent that uses XML tags.

    Args:
        tools: list of tools the agent can choose from
        llm_chain: The LLMChain to call to predict the next action

    Examples:

        .. code-block:: python

            from langchain.agents import XMLAgent
            from langchain

            tools = ...
            model =


    """

    tools: List[BaseTool]
    """List of tools this agent has access to."""
    llm_chain: LLMChain
    """Chain to use to predict action."""

    @property
    def input_keys(self) -> List[str]:
        return ["input"]

    @staticmethod
    def get_default_prompt() -> ChatPromptTemplate:
        base_prompt = ChatPromptTemplate.from_template(agent_instructions)
        return base_prompt + AIMessagePromptTemplate.from_template(
            "{intermediate_steps}"
        )

    @staticmethod
    def get_default_output_parser() -> XMLAgentOutputParser:
        return XMLAgentOutputParser()

    def plan(
        self,
        intermediate_steps: List[Tuple[AgentAction, str]],
        callbacks: Callbacks = None,
        **kwargs: Any,
    ) -> Union[AgentAction, AgentFinish]:
        log = ""
        for action, observation in intermediate_steps:
            log += (
                f"<tool>{action.tool}</tool><tool_input>{action.tool_input}"
                f"</tool_input><observation>{observation}</observation>"
            )
        tools = ""
        for tool in self.tools:
            tools += f"{tool.name}: {tool.description}\n"
        inputs = {
            "intermediate_steps": log,
            "tools": tools,
            "question": kwargs["input"],
            "stop": ["</tool_input>", "</final_answer>"],
        }
        response = self.llm_chain(inputs, callbacks=callbacks)
        return response[self.llm_chain.output_key]

    async def aplan(
        self,
        intermediate_steps: List[Tuple[AgentAction, str]],
        callbacks: Callbacks = None,
        **kwargs: Any,
    ) -> Union[AgentAction, AgentFinish]:
        log = ""
        for action, observation in intermediate_steps:
            log += (
                f"<tool>{action.tool}</tool><tool_input>{action.tool_input}"
                f"</tool_input><observation>{observation}</observation>"
            )
        tools = ""
        for tool in self.tools:
            tools += f"{tool.name}: {tool.description}\n"
        inputs = {
            "intermediate_steps": log,
            "tools": tools,
            "question": kwargs["input"],
            "stop": ["</tool_input>", "</final_answer>"],
        }
        response = await self.llm_chain.acall(inputs, callbacks=callbacks)
        return response[self.llm_chain.output_key]


def create_xml_agent(
    llm: BaseLanguageModel,
    tools: Sequence[BaseTool],
    prompt: BasePromptTemplate,
    tools_renderer: ToolsRenderer = render_text_description,
) -> Runnable:
    """Create an agent that uses XML to format its logic.

    Args:
        llm: LLM to use as the agent.
        tools: Tools this agent has access to.
        prompt: The prompt to use, must have input keys
            `tools`: contains descriptions for each tool.
            `agent_scratchpad`: contains previous agent actions and tool outputs.
        tools_renderer: This controls how the tools are converted into a string and
            then passed into the LLM. Default is `render_text_description`.

    Returns:
        A Runnable sequence representing an agent. It takes as input all the same input
        variables as the prompt passed in does. It returns as output either an
        AgentAction or AgentFinish.

    Example:

        .. code-block:: python

            from langchain import hub
            from langchain_community.chat_models import ChatAnthropic
            from langchain.agents import AgentExecutor, create_xml_agent

            prompt = hub.pull("hwchase17/xml-agent-convo")
            model = ChatAnthropic()
            tools = ...

            agent = create_xml_agent(model, tools, prompt)
            agent_executor = AgentExecutor(agent=agent, tools=tools)

            agent_executor.invoke({"input": "hi"})

            # Use with chat history
            from langchain_core.messages import AIMessage, HumanMessage
            agent_executor.invoke(
                {
                    "input": "what's my name?",
                    # Notice that chat_history is a string
                    # since this prompt is aimed at LLMs, not chat models
                    "chat_history": "Human: My name is Bob\\nAI: Hello Bob!",
                }
            )

    Prompt:

        The prompt must have input keys:
            * `tools`: contains descriptions for each tool.
            * `agent_scratchpad`: contains previous agent actions and tool outputs as an XML string.

        Here's an example:

        .. code-block:: python

            from langchain_core.prompts import PromptTemplate

            template = '''You are a helpful assistant. Help the user answer any questions.

            You have access to the following tools:

            {tools}

            In order to use a tool, you can use <tool></tool> and <tool_input></tool_input> tags. You will then get back a response in the form <observation></observation>
            For example, if you have a tool called 'search' that could run a google search, in order to search for the weather in SF you would respond:

            <tool>search</tool><tool_input>weather in SF</tool_input>
            <observation>64 degrees</observation>

            When you are done, respond with a final answer between <final_answer></final_answer>. For example:

            <final_answer>The weather in SF is 64 degrees</final_answer>

            Begin!

            Previous Conversation:
            {chat_history}

            Question: {input}
            {agent_scratchpad}'''
            prompt = PromptTemplate.from_template(template)
    """  # noqa: E501
    missing_vars = {"tools", "agent_scratchpad"}.difference(prompt.input_variables)
    if missing_vars:
        raise ValueError(f"Prompt missing required variables: {missing_vars}")

    prompt = prompt.partial(
        tools=tools_renderer(list(tools)),
    )
    llm_with_stop = llm.bind(stop=["</tool_input>"])

    agent = (
        RunnablePassthrough.assign(
            agent_scratchpad=lambda x: format_xml(x["intermediate_steps"]),
        )
        | prompt
        | llm_with_stop
        | XMLAgentOutputParser()
    )
    return agent



from langchain_community.document_loaders.unstructured import (
    UnstructuredAPIFileIOLoader,
    UnstructuredAPIFileLoader,
    UnstructuredBaseLoader,
    UnstructuredFileIOLoader,
    UnstructuredFileLoader,
    get_elements_from_api,
    satisfies_min_unstructured_version,
    validate_unstructured_version,
)

__all__ = [
    "satisfies_min_unstructured_version",
    "validate_unstructured_version",
    "UnstructuredBaseLoader",
    "UnstructuredFileLoader",
    "get_elements_from_api",
    "UnstructuredAPIFileLoader",
    "UnstructuredFileIOLoader",
    "UnstructuredAPIFileIOLoader",
]



from langchain_community.llms.azureml_endpoint import (
    AzureMLEndpointClient,
    AzureMLOnlineEndpoint,
    ContentFormatterBase,
    DollyContentFormatter,
    GPT2ContentFormatter,
    HFContentFormatter,
    LlamaContentFormatter,
    OSSContentFormatter,
)

__all__ = [
    "AzureMLEndpointClient",
    "ContentFormatterBase",
    "GPT2ContentFormatter",
    "OSSContentFormatter",
    "HFContentFormatter",
    "DollyContentFormatter",
    "LlamaContentFormatter",
    "AzureMLOnlineEndpoint",
]



import warnings
from typing import Any

from langchain_core._api import LangChainDeprecationWarning

from langchain.utils.interactive_env import is_interactive_env


def __getattr__(name: str) -> Any:
    from langchain_community import chat_message_histories

    # If not in interactive env, raise warning.
    if not is_interactive_env():
        warnings.warn(
            "Importing chat message histories from langchain is deprecated. Importing "
            "from langchain will no longer be supported as of gigachain==0.2.0. "
            "Please import from langchain-community instead:\n\n"
            f"`from langchain_community.chat_message_histories import {name}`.\n\n"
            "To install gigachain-community run `pip install -U gigachain-community`.",
            category=LangChainDeprecationWarning,
        )

    return getattr(chat_message_histories, name)


__all__ = [
    "AstraDBChatMessageHistory",
    "ChatMessageHistory",
    "CassandraChatMessageHistory",
    "CosmosDBChatMessageHistory",
    "DynamoDBChatMessageHistory",
    "ElasticsearchChatMessageHistory",
    "FileChatMessageHistory",
    "FirestoreChatMessageHistory",
    "MomentoChatMessageHistory",
    "MongoDBChatMessageHistory",
    "PostgresChatMessageHistory",
    "RedisChatMessageHistory",
    "RocksetChatMessageHistory",
    "SQLChatMessageHistory",
    "StreamlitChatMessageHistory",
    "SingleStoreDBChatMessageHistory",
    "XataChatMessageHistory",
    "ZepChatMessageHistory",
    "UpstashRedisChatMessageHistory",
    "Neo4jChatMessageHistory",
]



from langchain_community.graphs.networkx_graph import (
    KG_TRIPLE_DELIMITER,
    KnowledgeTriple,
    NetworkxEntityGraph,
    get_entities,
    parse_triples,
)

__all__ = [
    "KG_TRIPLE_DELIMITER",
    "KnowledgeTriple",
    "parse_triples",
    "get_entities",
    "NetworkxEntityGraph",
]



from langchain_community.example_selectors.ngram_overlap import (
    NGramOverlapExampleSelector,
    ngram_overlap_score,
)

__all__ = [
    "NGramOverlapExampleSelector",
    "ngram_overlap_score",
]



from langchain_community.utilities.clickup import (
    ClickupAPIWrapper,
    Component,
    CUList,
    Member,
    Space,
    Task,
    Team,
)

__all__ = [
    "Component",
    "Task",
    "CUList",
    "Member",
    "Team",
    "Space",
    "ClickupAPIWrapper",
]



from langchain_community.document_loaders.telegram import (
    TelegramChatApiLoader,
    TelegramChatFileLoader,
    concatenate_rows,
    text_to_docs,
)

__all__ = [
    "concatenate_rows",
    "TelegramChatFileLoader",
    "text_to_docs",
    "TelegramChatApiLoader",
]



from typing import List, Type, Union

from langchain_core.language_models import BaseLanguageModel
from langchain_core.prompts import ChatPromptTemplate
from langchain_core.pydantic_v1 import BaseModel
from langchain_core.runnables import Runnable
from langchain_core.utils.function_calling import convert_pydantic_to_openai_function

from langchain.output_parsers import PydanticToolsParser

_EXTRACTION_TEMPLATE = """Extract and save the relevant entities mentioned \
in the following passage together with their properties.

If a property is not present and is not required in the function parameters, do not include it in the output."""  # noqa: E501


def create_extraction_chain_pydantic(
    pydantic_schemas: Union[List[Type[BaseModel]], Type[BaseModel]],
    llm: BaseLanguageModel,
    system_message: str = _EXTRACTION_TEMPLATE,
) -> Runnable:
    """Creates a chain that extracts information from a passage.

    Args:
        pydantic_schemas: The schema of the entities to extract.
        llm: The language model to use.
        system_message: The system message to use for extraction.

    Returns:
        A runnable that extracts information from a passage.
    """
    if not isinstance(pydantic_schemas, list):
        pydantic_schemas = [pydantic_schemas]
    prompt = ChatPromptTemplate.from_messages(
        [("system", system_message), ("user", "{input}")]
    )
    functions = [convert_pydantic_to_openai_function(p) for p in pydantic_schemas]
    tools = [{"type": "function", "function": d} for d in functions]
    model = llm.bind(tools=tools)
    chain = prompt | model | PydanticToolsParser(tools=pydantic_schemas)
    return chain



"""**Graphs** provide a natural language interface to graph databases."""
import warnings
from typing import Any

from langchain_core._api import LangChainDeprecationWarning

from langchain.utils.interactive_env import is_interactive_env


def __getattr__(name: str) -> Any:
    from langchain_community import graphs

    # If not in interactive env, raise warning.
    if not is_interactive_env():
        warnings.warn(
            "Importing graphs from langchain is deprecated. Importing from "
            "langchain will no longer be supported as of langchain==0.2.0. "
            "Please import from langchain-community instead:\n\n"
            f"`from langchain_community.graphs import {name}`.\n\n"
            "To install gigachain-community run `pip install -U gigachain-community`.",
            category=LangChainDeprecationWarning,
        )

    return getattr(graphs, name)


__all__ = [
    "MemgraphGraph",
    "NetworkxEntityGraph",
    "Neo4jGraph",
    "NebulaGraph",
    "NeptuneGraph",
    "KuzuGraph",
    "HugeGraph",
    "RdfGraph",
    "ArangoGraph",
    "FalkorDBGraph",
]



from langchain_community.utils.ernie_functions import (
    FunctionDescription,
    ToolDescription,
    convert_pydantic_to_ernie_function,
    convert_pydantic_to_ernie_tool,
)

__all__ = [
    "FunctionDescription",
    "ToolDescription",
    "convert_pydantic_to_ernie_function",
    "convert_pydantic_to_ernie_tool",
]



from langchain_community.tools.plugin import (
    AIPlugin,
    AIPluginTool,
    AIPluginToolSchema,
    ApiConfig,
)

__all__ = [
    "ApiConfig",
    "AIPlugin",
    "AIPluginToolSchema",
    "AIPluginTool",
]



from langchain_community.callbacks.labelstudio_callback import (
    LabelStudioCallbackHandler,
    LabelStudioMode,
    get_default_label_configs,
)

__all__ = ["LabelStudioMode", "get_default_label_configs", "LabelStudioCallbackHandler"]



from langchain_community.vectorstores.sklearn import (
    BaseSerializer,
    BsonSerializer,
    JsonSerializer,
    ParquetSerializer,
    SKLearnVectorStore,
    SKLearnVectorStoreException,
)

__all__ = [
    "BaseSerializer",
    "JsonSerializer",
    "BsonSerializer",
    "ParquetSerializer",
    "SKLearnVectorStoreException",
    "SKLearnVectorStore",
]



from langchain_community.vectorstores.redis.schema import (
    FlatVectorField,
    HNSWVectorField,
    NumericFieldSchema,
    RedisDistanceMetric,
    RedisField,
    RedisModel,
    RedisVectorField,
    TagFieldSchema,
    TextFieldSchema,
    read_schema,
)

__all__ = [
    "RedisDistanceMetric",
    "RedisField",
    "TextFieldSchema",
    "TagFieldSchema",
    "NumericFieldSchema",
    "RedisVectorField",
    "FlatVectorField",
    "HNSWVectorField",
    "RedisModel",
    "read_schema",
]



from langchain_community.adapters.openai import (
    Chat,
    ChatCompletion,
    ChatCompletionChunk,
    ChatCompletions,
    Choice,
    ChoiceChunk,
    Completions,
    IndexableBaseModel,
    chat,
    convert_dict_to_message,
    convert_message_to_dict,
    convert_messages_for_finetuning,
    convert_openai_messages,
)

__all__ = [
    "IndexableBaseModel",
    "Choice",
    "ChatCompletions",
    "ChoiceChunk",
    "ChatCompletionChunk",
    "convert_dict_to_message",
    "convert_message_to_dict",
    "convert_openai_messages",
    "ChatCompletion",
    "convert_messages_for_finetuning",
    "Completions",
    "Chat",
    "chat",
]



"""Implementations of key-value stores and storage helpers.

Module provides implementations of various key-value stores that conform
to a simple key-value interface.

The primary goal of these storages is to support implementation of caching.
"""
import warnings
from typing import Any

from langchain_core._api import LangChainDeprecationWarning

from langchain.storage._lc_store import create_kv_docstore, create_lc_store
from langchain.storage.encoder_backed import EncoderBackedStore
from langchain.storage.file_system import LocalFileStore
from langchain.storage.in_memory import InMemoryByteStore, InMemoryStore
from langchain.utils.interactive_env import is_interactive_env


def __getattr__(name: str) -> Any:
    from langchain_community import storage

    # If not in interactive env, raise warning.
    if not is_interactive_env():
        warnings.warn(
            "Importing stores from langchain is deprecated. Importing from "
            "langchain will no longer be supported as of langchain==0.2.0. "
            "Please import from langchain-community instead:\n\n"
            f"`from langchain_community.storage import {name}`.\n\n"
            "To install gigachain-community run `pip install -U gigachain-community`.",
            category=LangChainDeprecationWarning,
        )

    return getattr(storage, name)


__all__ = [
    "EncoderBackedStore",
    "InMemoryStore",
    "InMemoryByteStore",
    "LocalFileStore",
    "RedisStore",
    "create_lc_store",
    "create_kv_docstore",
    "UpstashRedisByteStore",
    "UpstashRedisStore",
]



from langchain_community.document_loaders.parsers.audio import (
    OpenAIWhisperParser,
    OpenAIWhisperParserLocal,
    YandexSTTParser,
)

__all__ = ["OpenAIWhisperParser", "OpenAIWhisperParserLocal", "YandexSTTParser"]



from langchain_community.vectorstores.azure_cosmos_db import (
    AzureCosmosDBVectorSearch,
    CosmosDBDocumentType,
    CosmosDBSimilarityType,
)

__all__ = [
    "CosmosDBSimilarityType",
    "CosmosDBDocumentType",
    "AzureCosmosDBVectorSearch",
]
